@@ -1,6 +1,6 @@
 #
 # Authentic Theme (https://github.com/authentic-theme/authentic-theme)
-# Copyright Ilia Rostovtsev <programming@rostovtsev.io>
+# Copyright Ilia Rostovtsev <ilia@virtualmin.com>
 # Licensed under MIT (https://github.com/authentic-theme/authentic-theme/blob/master/LICENSE)
 #
 use strict;
@@ -11,6 +11,7 @@ our (%gconfig,
      $current_theme,
      $theme_webprefix,
      %theme_config,
+     $theme_info,
      %theme_text,
      $has_usermin,
      $has_usermin_root_dir,
@@ -18,7 +19,8 @@ our (%gconfig,
 
 sub theme_settings_raw
 {
-    return (
+    # Available settings
+    my @theme_settings_raw = (
         [
          {  'id'    => 's1',
             'title' => &theme_text('settings_global_general_options_title'),
@@ -29,13 +31,14 @@ sub theme_settings_raw
                        'settings_right_virtualmin_default',
                        'settings_right_cloudmin_default',
                        'settings_right_page_keep',
-                       'settings_right_reload',
+                       'settings_right_sync',
                        'settings_document_title',
-                       'settings_cm_editor_palette',
+                       'settings_palette_auto',
                        'settings_global_palette_unauthenticated',
+                       'settings_cm_editor_palette',
                        'settings_theme_config_admins_only_privileged',
-            ]
-         }
+                       'settings_embed_product_splash_privileged',
+            ] }
         ],
 
         [
@@ -49,11 +52,7 @@ sub theme_settings_raw
                        'settings_sysinfo_hidden_panels_user',
                        'settings_sysinfo_max_servers',
                        'settings_sysinfo_real_time_status',
-                       'settings_sysinfo_real_time_status_disk',
-                       'settings_sysinfo_real_time_stored',
-                       'settings_sysinfo_real_time_stored_length',
-            ]
-         }
+            ] }
         ],
 
         [
@@ -70,6 +69,7 @@ sub theme_settings_raw
                        'settings_contrast_level_navigation',
                        'settings_navigation_always_collapse',
                        'settings_leftmenu_width',
+                       'settings_auto_open_panels_on_proximity',
                        'settings_switch_rdisplay',
                        'settings_navigation_auto_fold_category',
                        'settings_leftmenu_section_hide_refresh_modules',
@@ -77,6 +77,7 @@ sub theme_settings_raw
                        'settings_collapse_navigation_link',
                        'settings_sysinfo_link_mini',
                        'settings_show_night_mode_link',
+                       'settings_show_terminal_link',
                        'settings_show_terminal_link2',
                        'settings_favorites',
                        'settings_theme_options_button',
@@ -87,8 +88,7 @@ sub theme_settings_raw
                        'settings_leftmenu_user_html',
                        'settings_leftmenu_user_html_privileged',
                        'settings_leftmenu_custom_links',
-            ]
-         }
+            ] }
         ],
 
         [
@@ -100,8 +100,7 @@ sub theme_settings_raw
                        'settings_side_slider_sysinfo_enabled',
                        'settings_side_slider_notifications_enabled',
                        'settings_side_slider_favorites_enabled',
-            ]
-         }
+            ] }
         ],
 
         [
@@ -109,10 +108,11 @@ sub theme_settings_raw
             'title' => &theme_text('settings_right_table_options'),
             'data'  => [
                        'settings_right_table_links_type',
+                       'settings_right_table_links_sorted',
                        'settings_right_table_animate_icons',
                        'settings_right_table_grayscaled_icons',
-            ]
-         }
+                       'settings_right_table_layout_control',
+            ] }
         ],
 
         [
@@ -121,22 +121,23 @@ sub theme_settings_raw
             'data'  => [
                        'settings_hotkeys_active',
                        'settings_hotkey_toggle_modifier',
+                       'settings_hotkey_toggle_hold_modifier',
                        'settings_hotkey_toggle_key_webmin',
                        'settings_hotkey_toggle_key_virtualmin',
                        'settings_hotkey_toggle_key_cloudmin',
                        'settings_hotkey_toggle_key_usermin',
                        'settings_hotkey_toggle_key_webmail',
+                       'settings_hotkey_shell',
                        'settings_hotkey_shell2',
                        'settings_hotkey_sysinfo',
-                       'settings_hotkey_toggle_slider',
                        'settings_hotkey_favorites',
-                       'settings_hotkey_focus_search',
-                       'settings_hotkey_reload',
                        'settings_hotkey_navigation',
                        'settings_hotkey_slider',
                        'settings_hotkey_toggle_key_night_mode',
-            ]
-         }
+                       'settings_hotkey_focus_search',
+                       'settings_hotkey_reload',
+                       'settings_hotkey_logout_dbl',
+            ] }
         ],
 
         [
@@ -153,17 +154,22 @@ sub theme_settings_raw
                        'settings_hotkey_custom_7_user',
                        'settings_hotkey_custom_8_user',
                        'settings_hotkey_custom_9_user',
-            ]
-         }
-        ],
-
+            ] }
+        ]);
+    # Add upgrade settings if available
+    if ($theme_config{'settings_upgrade_allowed'} eq 'true') {
+        push(@theme_settings_raw,
         [
          {  'id'    => 's8',
             'title' => &theme_text('settings_right_soft_updates_page_options'),
-            'data'  =>
-              ['settings_sysinfo_theme_updates', 'settings_cache_interval', 'settings_sysinfo_theme_updates_for_usermin',]
-         }
+            'data'  => [
+                        'settings_sysinfo_theme_updates', 
+                        'settings_sysinfo_theme_updates_for_usermin',
+                        'settings_cache_interval'
+            ] }
         ]);
+    }
+    return @theme_settings_raw;
 }
 
 sub theme_settings_filter
@@ -175,8 +181,7 @@ sub theme_settings_filter
         !foreign_available("virtual-server"))
     {
         push(@theme_settings_filter,
-             'settings_right_default_tab_webmin',
-             'settings_right_reload', 'settings_right_page_keep');
+             'settings_right_default_tab_webmin', 'settings_right_page_keep');
     }
 
     # Assign default for options provided by the server side
@@ -236,17 +241,22 @@ sub theme_settings_filter
         push(@theme_settings_filter, 'settings_leftmenu_netdata', 'settings_leftmenu_netdata_link');
     }
 
-    # Exclude Command Shell button if not available
-    if (!foreign_available("shell")) {
+    # Exclude Command Shell button if not available or if Terminal is available
+    if (!foreign_available("shell") || foreign_available("xterm")) {
         push(@theme_settings_filter, 'settings_show_terminal_link2', 'settings_hotkey_shell2');
     }
 
+    # Exclude Terminal button if not available
+    if (!foreign_available("xterm")) {
+        push(@theme_settings_filter, 'settings_show_terminal_link', 'settings_hotkey_shell');
+    }
+
     # Limit to certain options for non privleged user
-    if ($get_user_level ne '0') {
+    if (!&webmin_user_is_admin()) {
         push(@theme_settings_filter,
              'settings_theme_config_admins_only_privileged',
+             'settings_embed_product_splash_privileged',
              'settings_hotkey_slider',
-             'settings_hotkey_toggle_slider',
              'settings_global_palette_unauthenticated',
              'settings_sysinfo_easypie_charts',
              'settings_sysinfo_easypie_charts_size',
@@ -254,9 +264,7 @@ sub theme_settings_filter
              'settings_sysinfo_easypie_charts_scale',
              'settings_sysinfo_max_servers',
              'settings_sysinfo_real_time_status',
-             'settings_sysinfo_real_time_status_disk',
-             'settings_sysinfo_real_time_stored',
-             'settings_sysinfo_real_time_stored_length',
+             'settings_sysinfo_real_time_stored_duration',
              'settings_leftmenu_section_hide_refresh_modules',
              'settings_leftmenu_section_hide_unused_modules',
              'settings_leftmenu_netdata',
@@ -268,10 +276,10 @@ sub theme_settings_filter
              'settings_side_slider_notifications_enabled',
              'settings_side_slider_favorites_enabled',
              'settings_sysinfo_theme_updates',
-             'settings_cache_interval',
-             'settings_sysinfo_theme_updates_for_usermin');
+             'settings_sysinfo_theme_updates_for_usermin',
+             'settings_cache_interval');
         if ($get_user_level eq '3') {
-            push(@theme_settings_filter, 'settings_hotkey_toggle_key_webmin', 'settings_webmin_default_module');
+            push(@theme_settings_filter, 'settings_hotkey_toggle_key_webmin');
         } elsif ($get_user_level eq '2' || $get_user_level eq '4') {
             push(@theme_settings_filter,
                  'settings_hotkey_toggle_key_webmin',
@@ -303,7 +311,7 @@ sub theme_settings_format
         }
     }
 
-    my $v = (length $theme_config{$k} ? $theme_config{$k} : $v);
+    $v = (length $theme_config{$k} ? $theme_config{$k} : $v);
 
     if ($v eq 'true' || $v eq 'false') {
         $v = ui_yesno_radio($k, $v, 'true', 'false');
@@ -335,8 +343,9 @@ sub theme_settings_format
              $k eq 'settings_hotkey_focus_search'  ||
              $k eq 'settings_hotkey_navigation'    ||
              $k eq 'settings_hotkey_slider'        ||
-             $k eq 'settings_hotkey_toggle_slider' ||
              $k eq 'settings_hotkey_reload'        ||
+             $k eq 'settings_hotkey_logout_dbl'    ||
+             $k eq 'settings_hotkey_shell'         ||
              $k eq 'settings_hotkey_shell2'        ||
              $k eq 'settings_hotkey_sysinfo'       ||
              $k eq 'settings_hotkey_favorites')
@@ -390,12 +399,28 @@ sub theme_settings_format
         }
         $v = '
                 <input style="display: inline; width: ' .
-          $iwidth . '%; height: 28px; vertical-align: middle;" class="form-control ui_textbox" type="range" min="' .
-          $range_min . '" max="' . $range_max . '" step="' . $range_step . '" name="' . $k . '" value="' . $v . '">
+          $iwidth .
+          '%; height: 28px; vertical-align: middle;" class="form-control ui_textbox" type="range" min="' .
+          $range_min .
+          '" max="' .
+          $range_max .
+          '" step="' .
+          $range_step .
+          '" name="' .
+          $k .
+          '" value="' .
+          $v . '">
             ';
 
     } elsif ($k eq 'settings_leftmenu_custom_links') {
-        $v = ui_textarea($k, $v, 1);
+        my $line_count = 1;
+        if ($v) {
+            $v = replace('\'', '"', un_urlize($v, 1));
+            $v = convert_from_json($v);
+            $v = convert_to_json($v, 1);
+            $line_count = () = $v =~ /\n/g;
+        }
+        $v = ui_textarea($k, $v, $line_count);
     } elsif ($k =~ /settings_hotkey_custom/ ||
              $k eq 'settings_leftmenu_netdata_link' ||
              $k eq 'settings_leftmenu_user_html')
@@ -409,8 +434,11 @@ sub theme_settings_format
         $v =~ s/(<(\/|\s*)(html|head|meta|link|title|body).*?>)//g;
         $v = '
                 <input style="display: inline;'
-          . $width . 'height: 28px; vertical-align: middle;" class="form-control ui_textbox" type="text" name="' .
-          $k . '" value="' . $v . '">
+          . $width .
+          'height: 28px; vertical-align: middle;" class="form-control ui_textbox" type="text" name="' .
+          $k .
+          '" value="' .
+          $v . '">
             ';
     } elsif ($k eq 'settings_right_default_tab_webmin') {
         $v = '<select class="ui_select" name="' . $k . '">
@@ -420,25 +448,41 @@ sub theme_settings_format
                 '
           . (&foreign_available("virtual-server") &&
              ' <option value="virtualmin"' .
-             ($v eq 'virtualmin' && ' selected') . '>' . $theme_text{'theme_xhred_titles_vm'} . '</option> ') .
+             ($v eq 'virtualmin' && ' selected') . '>' .
+             $theme_text{'theme_xhred_titles_vm'} .
+             '</option> ') .
           '
 
                '
           . (&foreign_available("server-manager") &&
              ' <option value="cloudmin"' .
-             ($v eq 'cloudmin' && ' selected') . '>' . $theme_text{'theme_xhred_titles_cm'} . '</option>') .
+             ($v eq 'cloudmin' && ' selected') . '>' .
+             $theme_text{'theme_xhred_titles_cm'} .
+             '</option>') .
           '
                 </select>';
     } elsif ($k eq 'settings_webmin_default_module') {
-        $v = settings_get_select_default_module('goto_webmin_default_module', $gconfig{'gotomodule'});
+        if ($get_user_level eq '3') {
+            $k = 'settings_usermin_default_module';
+        }
+        $v = settings_get_select_default_module('goto_webmin_default_module', get_default_module());
     } elsif ($k eq 'settings_right_default_tab_usermin') {
         $v = '<select class="ui_select" name="' . $k . '">
                 <option value="/"'
           . ($v eq '/' && ' selected') . '>' . $theme_text{'theme_xhred_titles_um'} . '</option>       
                 <option value="webmail"' .
-          ($v eq 'webmail' && ' selected') . '>' . $theme_text{'theme_xhred_titles_mail'} . '</option>
+          ($v eq 'webmail' && ' selected') . '>' .
+          $theme_text{'theme_xhred_titles_mail'} .
+          '</option>
           
 
+                </select>';
+    } elsif ($k eq 'settings_hotkey_toggle_hold_modifier') {
+        $v = '<select class="ui_select" name="' . $k . '">
+                    <option value="altKey"'
+          . ($v eq 'altKey' && ' selected') . '>Alt</option>
+                    <option value="ctrlKey"'
+          . ($v eq 'ctrlKey' && ' selected') . '>Ctrl</option>
                 </select>';
     } elsif ($k eq 'settings_hotkey_toggle_modifier') {
         $v = '<select class="ui_select" name="' . $k . '">
@@ -452,23 +496,41 @@ sub theme_settings_format
     } elsif ($k eq 'settings_cache_interval') {
         $v = '<select class="ui_select" name="' . $k . '">
                     <option value="3600"' .
-          ($v eq '3600' && ' selected') . '>' . $theme_text{'settings_cache_interval_1h'} . '</option>
+          ($v eq '3600' && ' selected') . '>' .
+          $theme_text{'settings_cache_interval_1h'} .
+          '</option>
                     <option value="43200"' .
-          ($v eq '43200' && ' selected') . '>' . $theme_text{'settings_cache_interval_12h'} . '</option>
+          ($v eq '43200' && ' selected') . '>' .
+          $theme_text{'settings_cache_interval_12h'} .
+          '</option>
                     <option value="86400"' .
-          ($v eq '86400' && ' selected') . '>' . $theme_text{'settings_cache_interval_1d'} . '</option>
+          ($v eq '86400' && ' selected') . '>' .
+          $theme_text{'settings_cache_interval_1d'} .
+          '</option>
                     <option value="604800"' .
-          ($v eq '604800' && ' selected') . '>' . $theme_text{'settings_cache_interval_7d'} . '</option>
+          ($v eq '604800' && ' selected') . '>' .
+          $theme_text{'settings_cache_interval_7d'} .
+          '</option>
                     <option value="1209600"' .
-          ($v eq '1209600' && ' selected') . '>' . $theme_text{'settings_cache_interval_14d'} . '</option>
+          ($v eq '1209600' && ' selected') . '>' .
+          $theme_text{'settings_cache_interval_14d'} .
+          '</option>
                     <option value="2419200"' .
-          ($v eq '2419200' && ' selected') . '>' . $theme_text{'settings_cache_interval_1m'} . '</option>
+          ($v eq '2419200' && ' selected') . '>' .
+          $theme_text{'settings_cache_interval_1m'} .
+          '</option>
                     <option value="7257600"' .
-          ($v eq '7257600' && ' selected') . '>' . $theme_text{'settings_cache_interval_3m'} . '</option>
+          ($v eq '7257600' && ' selected') . '>' .
+          $theme_text{'settings_cache_interval_3m'} .
+          '</option>
                     <option value="14515200"' .
-          ($v eq '14515200' && ' selected') . '>' . $theme_text{'settings_cache_interval_6m'} . '</option>
+          ($v eq '14515200' && ' selected') . '>' .
+          $theme_text{'settings_cache_interval_6m'} .
+          '</option>
                     <option value="29030400"' .
-          ($v eq '29030400' && ' selected') . '>' . $theme_text{'settings_cache_interval_1y'} . '</option>
+          ($v eq '29030400' && ' selected') . '>' .
+          $theme_text{'settings_cache_interval_1y'} .
+          '</option>
                 </select>';
     } elsif ($k eq 'settings_right_virtualmin_default') {
         if (foreign_available('virtual-server')) {
@@ -478,7 +540,7 @@ sub theme_settings_format
                              ['index.cgi', $theme_text{'theme_config_virtualmin'}],
                              map    {[$_->{'id'}, &virtual_server::show_domain_name($_)]}
                                grep {&virtual_server::can_edit_domain($_)}
-                               sort {$a->{'dom'} cmp $b->{'dom'}} &virtual_server::list_domains()
+                               sort {$a->{'dom'} cmp $b->{'dom'}} &virtual_server::list_visible_domains()
                             ]);
         }
     } elsif ($k eq 'settings_right_cloudmin_default') {
@@ -494,48 +556,43 @@ sub theme_settings_format
 
     } elsif ($k eq 'settings_navigation_color') {
         $v = settings_get_select_navigation_color($v, $k);
-    } elsif ($k eq 'settings_cm_editor_palette') {
-        $v = settings_get_select_editor_color($v, $k);
     } elsif ($k eq 'settings_global_palette_unauthenticated') {
         $v = ui_select($k,
                        $v,
-                       [[('light', $theme_text{'theme_xhred_global_light'})],
+                       [[('auto', $theme_text{'theme_xhred_global_auto'})],
+                        [('light', $theme_text{'theme_xhred_global_light'})],
                         [('dark',  $theme_text{'theme_xhred_global_dark'})]
                        ]);
-    } elsif ($k eq 'settings_sysinfo_real_time_stored_length') {
-        $v = '<select class="ui_select" name="' . $k . '">
-
-                    <option value="600"'
-          . ($v eq '600' && ' selected') . '>10 ' . lc($theme_text{'theme_xhred_global_minutes'}) . '</option>
-
-                    <option value="1800"'
-          . ($v eq '1800' && ' selected') . '>30 ' . lc($theme_text{'theme_xhred_global_minutes'}) . '</option>
-
-              <option value="3600"'
-          . ($v eq '3600' && ' selected') . '>1 ' . lc($theme_text{'theme_xhred_global_hour'}) . '</option>
-
-              <option value="7200"'
-          . ($v eq '7200' && ' selected') . '>2 ' . lc($theme_text{'theme_xhred_global_hours'}) . '</option>
-
-              <option value="10800"'
-          . ($v eq '10800' && ' selected') . '>3 ' . lc($theme_text{'theme_xhred_global_hours'}) . '</option>
-
-              <option value="21600"'
-          . ($v eq '21600' && ' selected') . '>6 ' . lc($theme_text{'theme_xhred_global_hours'}) . '</option>
-
-              <option value="43200"'
-          . ($v eq '43200' && ' selected') . '>12 ' . lc($theme_text{'theme_xhred_global_hours'}) . '</option>
-
-              <option value="86400"'
-          . ($v eq '86400' && ' selected') . '>24 ' . lc($theme_text{'theme_xhred_global_hours'}) . '</option>
-
-                </select>';
+    } elsif ($k eq 'settings_cm_editor_palette') {
+        $v = settings_get_select_editor_color($v, $k);
+    } elsif ($k eq 'settings_right_sync') {
+        $v = ui_select($k, $v, [
+            [2, $theme_text{"settings_right_sync2"}],
+            [1, $theme_text{"settings_right_sync1"}],
+            [0, $theme_text{"settings_right_sync0"}]
+        ]);
     } elsif ($k eq 'settings_document_title') {
         $v = settings_get_select_document_title($v, $k);
     } elsif ($k eq 'settings_sysinfo_real_time_status') {
-        my $yes_forced =
-"$theme_text{'settings_sysinfo_real_time_status_forced'} <sup @{[get_button_tooltip('settings_sysinfo_real_time_status_forced_warn')]} class=\"fa fa-exclamation-circle\"></sup>";
-        $v = ui_radio($k, $v, [[1, $text{'yes'}], [2, $yes_forced], [0, $text{'no'}]]);
+        my $realtime = 'settings_sysinfo_real_time_';
+        my $realtime_pref = "${realtime}status_";
+        my $duration_key = "${realtime}stored_duration";
+        my $select = ui_select($duration_key, $theme_config{$duration_key},
+                       [
+                        [(900, $theme_text{"${realtime_pref}history_duration1"})],
+                        [(1800, $theme_text{"${realtime_pref}history_duration2"})],
+                        [(2700, $theme_text{"${realtime_pref}history_duration3"})],
+                        [(3600, $theme_text{"${realtime_pref}history_duration4"})],
+                        [(21600, $theme_text{"${realtime_pref}history_duration5"})],
+                        [(43200, $theme_text{"${realtime_pref}history_duration6"})],
+                        [(64800, $theme_text{"${realtime_pref}history_duration7"})],
+                        [(86400, $theme_text{"${realtime_pref}history_duration8"})]
+                       ]);
+        $v = ui_radio($k, $v, [
+            [1, $theme_text{"${realtime_pref}history1"}.$select],
+            [2, $theme_text{"${realtime_pref}history2"}],
+            [0, $text{'no'}]
+        ]);
     } elsif ($k eq 'settings_right_table_links_type') {
         $v = ui_radio($k,
                       $v,
@@ -544,35 +601,47 @@ sub theme_settings_format
                        [0, $theme_text{'settings_right_table_links_type_0'}]
                       ]);
     }
-    my $description     = $theme_text{ $k . '_description' };
+    my $description = $theme_text{ $k . '_description' };
 
     # Return formatted
+    if ($description =~ /<pre.*?data-json>(.*?)<\/pre>/s) {
+        my $json_str = $1;
+        my $json_data = convert_from_json($json_str);
+        my $pretty_json = convert_to_json($json_data, 1);
+        $pretty_json =~ s/   /&nbsp;/g;
+        $pretty_json =~ s/\n/<br>/g;
+        $description =~ s/<pre.*?data-json>.*?<\/pre>/<pre data-json>$pretty_json<\/pre>/s;
+    }
     return [
-            (
-             (
-              $description && (
-                             $k =~ /level_navigation|leftmenu_width/ ? undef :
-                             '<sup class="fa fa-fw fa-0_80x fa-question-circle module-help showpass-popover cursor-pointer tconfig-popover" data-html="true" data-toggle="popover" data-trigger="click" data-title="' . $theme_text{$k} . '" data-content="' . html_escape($description) . '"></sup>'
-              )
-             ) .
-               '<span data-text>' . $theme_text{$k} . '</span>'
-               .
-               (
-                 $description && ($k =~ /level_navigation|leftmenu_width/ ?
-                                  '<div class="smaller text-normal no-padding">' . $description . '</div>' :
-                                  $k =~ /sysinfo_theme_updates/ &&
-                                  '<div class="smaller text-normal no-padding margined-left-1"></div>'
-                 )
-               )
-            ),
-            "<span>$v</span>"];
+        (
+         (
+          $description && (
+              $k =~ /level_navigation|leftmenu_width/ ? undef :
+'<sup class="fa fa-fw fa-0_80x fa-question-circle module-help showpass-popover cursor-pointer tconfig-popover" data-html="true" data-toggle="popover" data-trigger="click" data-title="'
+              . $theme_text{$k}
+              . '" data-content="' . html_escape($description) . '"></sup>')
+         ) .
+           '<span data-text>' .
+           $theme_text{$k} .
+           '</span>'
+           .
+           (
+             $description && (
+                       $k =~ /level_navigation|leftmenu_width/ ?
+                       '<div class="smaller text-normal no-padding">' . $description . '</div>' :
+                       $k =~ /sysinfo_theme_updates/ &&
+                       '<div class="smaller text-normal no-padding margined-left-1 theme-version-installed-vs-remote"></div>'
+             ))
+        ),
+        "<span>$v</span>"];
 }
 
 sub theme_controls
 {
-    my ($section)                 = @_;
-    my $changelog_contents        = read_file_contents($root_directory . '/' . $current_theme . "/CHANGELOG.md");
-    my @changelog_stable_versions = ($changelog_contents =~ /####\s+Version\s+((?|\d+.\d+\s+|\d+.\d+.\d+\s+)\([\d\w\s,]+\))/g);
+    my ($section) = @_;
+    my $changelog_contents = read_file_contents($root_directory . '/' . $current_theme . "/CHANGELOG.md");
+    my @changelog_stable_versions =
+      ($changelog_contents =~ /####\s+Version\s+((?|\d+.\d+\s+|\d+.\d+.\d+\s+)\([\d\w\s,]+\))/g);
     my $stable_versions_dropdown_submenu;
     my $stable_versions_dropdown_submenu_content;
     if (@changelog_stable_versions) {
@@ -582,17 +651,19 @@ sub theme_controls
             my ($ver_str) = ($ver =~ /^(?|(\d+.\d+)\s+|(\d+.\d+.\d+)\s+)/);
             $stable_versions_dropdown_submenu_content .=
               '<li><a tabindex="-1" href="javascript:;" data-git="1" data-stable="1" data-version="' .
-              $ver_str . '" class="authentic_update">&nbsp;' . $ver . '</a></li>';
+              $ver_str .
+              '" class="authentic_update">&nbsp;' .
+              $ver .
+              '</a></li>';
         }
         $stable_versions_dropdown_submenu_content .= '</ul>';
     }
 
     my $update_dropdown = (
-        ($get_user_level eq '0' && $section eq $theme_text{'settings_right_soft_updates_page_options'}) ?
+        (&webmin_user_is_admin() && $section eq $theme_text{'settings_right_soft_updates_page_options'}) ?
           '                     <span id="force_update_menu_cnt" class="dropup"'
           .
-          ( has_command('git') ?
-              get_button_tooltip('settings_update_theme_tooltip', undef, undef, 1, 1, '#force_update_menu_cnt') :
+          ( has_command('git') ? '' :
               get_button_tooltip('settings_sysinfo_theme_updates_description', undef, undef, 1, 1)
           ) .
           '>
@@ -604,10 +675,12 @@ sub theme_controls
                                          <span class="caret"></span>
                                        </button>
                                        <ul class="dropdown-menu" aria-labelledby="force_update_menu">
-                                         <li' . $stable_versions_dropdown_submenu .
+                                         <li' .
+          $stable_versions_dropdown_submenu .
 '><a data-git="1" data-stable="1" class="authentic_update" href="javascript:;"><i class="fa2 fa2-release-tagged fa-0_90x"></i>'
           . $theme_text{'theme_xhred_force_upgrade_stable'}
-          . '</a>' . $stable_versions_dropdown_submenu_content . '</li>
+          . '</a>' .
+          $stable_versions_dropdown_submenu_content . '</li>
                                          <li><a data-git="1" data-stable="0" class="authentic_update" href="javascript:;"><i class="fa2 fa2-release-master"></i>'
           . $theme_text{'theme_xhred_force_upgrade_beta'} . '</a></li>
                                        </ul>
@@ -622,12 +695,12 @@ sub theme_controls
             <a tabindex='1' class=\"btn btn-default capitalize\" id=\"atrestore\">
                 <i class=\"fa fa-fw fa-history\"></i><span>$theme_text{'settings_right_restore_defaults'}</span>
             </a>
-            <a tabindex='1' class=\"btn btn-default capitalize\" onclick=\"theme_cache_clear(this);\" @{[get_button_tooltip('settings_reset_cache_tooltip', undef, undef, 1, 1)]}>
+            <a tabindex='1' class=\"btn btn-default capitalize\" onclick=\"theme_cache_clear(this,1);\">
                 <i class=\"fa fa-fw fa-hourglass-o\"></i><span>$theme_text{'settings_right_clear_local_cache'}</span>
             </a>
             $update_dropdown
         </div>",
-        ($get_user_level eq '0' ?
+        (&webmin_user_is_admin() ?
            "<div class=\"btn-group\">
             <a tabindex='1' class=\"btn btn-default page_footer_ajax_submit capitalize\" id=\"edit_logos\" href=\"$theme_webprefix/settings-backgrounds.cgi\">
                 <i class=\"fa fa-fw fa-image\"></i><span>$theme_text{'theme_xhred_settings_right_theme_bgs'}</span>
@@ -651,27 +724,33 @@ sub settings_get_select_navigation_color
                     <option value="blue"'
       . ($v eq 'blue' && ' selected') . '>Blue (' . $theme_text{'theme_xhred_global_default'} . ')</option>
 
+                    <option value="teal"'
+      . ($v eq 'teal' && ' selected') . '>Teal</option>
+
+                    <option value="green"'
+      . ($v eq 'green' && ' selected') . '>Green</option>
+
+                    <option value="purple"'
+      . ($v eq 'purple' && ' selected') . '>Purple</option>
+      
                     <option value="brown"'
       . ($v eq 'brown' && ' selected') . '>Brown</option>
 
                     <option value="gold"'
       . ($v eq 'gold' && ' selected') . '>Gold</option>
 
-                    <option value="green"'
-      . ($v eq 'green' && ' selected') . '>Green</option>
-
-                    <option value="grey"'
-      . ($v eq 'grey' && ' selected') . '>Gray</option>
-
                     <option value="orange"'
       . ($v eq 'orange' && ' selected') . '>Orange</option>
       
-                    <option value="purple"'
-      . ($v eq 'purple' && ' selected') . '>Purple</option>
-      
-                 <option value="red"'
+                     <option value="red"'
       . ($v eq 'red' && ' selected') . '>Red</option>
 
+                     <option value="maroon"'
+      . ($v eq 'maroon' && ' selected') . '>Maroon</option>
+
+                    <option value="grey"'
+      . ($v eq 'grey' && ' selected') . '>Gray</option>
+
                     <option value="white"'
       . ($v eq 'white' && ' selected') . '>White</option>
 
@@ -686,6 +765,9 @@ sub settings_get_select_editor_color
     my ($v, $k) = @_;
     return '<select class="ui_select" name="' . $k . '">
 
+            <option value="auto"'
+      . ($v eq 'auto' && ' selected') . '>' . $theme_text{'theme_xhred_global_auto'} . '</option>
+
             <option value="monokai"'
       . ($v eq 'monokai' && ' selected') . '>' . $theme_text{'theme_xhred_global_dark'} . '</option>
 
@@ -708,8 +790,10 @@ sub settings_get_select_document_title
       <option value="7"'
       . ($v eq '7' && ' selected') . '>' . theme_text('settings_document_title_option_7', ucfirst($prod_name)) . '</option>
       <option value="1"'
-      . ($v eq '1' && ' selected') . '>' . theme_text('settings_document_title_option_1', ucfirst($prod_name)) .
-      ' (' . $theme_text{'theme_xhred_global_default'} . ')</option>
+      . ($v eq '1' && ' selected') . '>' .
+      theme_text('settings_document_title_option_1', ucfirst($prod_name)) . ' (' .
+      $theme_text{'theme_xhred_global_default'} .
+      ')</option>
       <option value="2"'
       . ($v eq '2' && ' selected') . '>' . theme_text('settings_document_title_option_2', ucfirst($prod_name)) . '</option>
       <option value="4"'
@@ -731,6 +815,9 @@ sub settings_get_select_default_module
     my ($name, $value) = @_;
     my @modules = get_available_module_infos();
     @modules = grep {!$_->{'hidden'} && !$_->{'webmin_hidden'}} @modules;
+    if ($get_user_level eq '3') {
+        @modules = grep {$_->{'dir'} ne 'mailbox'} @modules;
+    }
     my $select = ui_select($name,
                            $value,
                            [["", $theme_text{'theme_xhred_titles_dashboard'}],
@@ -740,4 +827,56 @@ sub settings_get_select_default_module
     return $select;
 }
 
+sub theme_settings_data
+{
+    my @settings         = theme_settings_raw();
+    my @excluded_options = theme_settings_filter();
+    my @sections;
+    my @config_quick_access;
+
+    # Format options for display and build quick access filter
+    foreach my $sections (0 .. $#settings) {
+        foreach my $section ($settings[$sections]) {
+            for (my $i = 0; $i < scalar(@{ $section->[0]->{'data'} }); $i++) {
+                my $key_value = @{ $section->[0]->{'data'} }[$i];
+                my @key_value_formated =
+                  theme_settings_format(@{ $section->[0]->{'data'} }[$i],
+                                        $theme_config{ @{ $section->[0]->{'data'} }[$i] },
+                                        \@excluded_options);
+                if (!$key_value_formated[0][0]) {
+                    delete $section->[0]->{'data'}[$i];
+                    next;
+                }
+                $section->[0]->{'data'}[$i] = [$key_value_formated[0][0], $key_value_formated[0][1]];
+                $key_value_formated[0][0] =~ s/<div.*?>.*?<\/div>//gm;
+                $key_value_formated[0][0] =~ s/<span\s+data-text.*?>(.*?)<\/span>/$1/gm;
+                $key_value_formated[0][0] =~ s/<span.*?>.*?<\/span>//gm;
+                $key_value_formated[0][0] =~ s/<sup.*?>.*?<\/sup>//gm;
+                $key_value_formated[0][0] =~ s/<code>(.*?)<\/code>.*/$1/;
+                $key_value_formated[0][0] = entities_to_ascii($key_value_formated[0][0]);
+                push(@config_quick_access,
+                     {  'value'   => $key_value_formated[0][0],
+                        'key'     => $key_value,
+                        'section' => $section->[0]->{'id'},
+                        'data'    => { category => $section->[0]->{'title'} }
+                     });
+            }
+
+            # Remove undefined values which where previously discarded
+            @{ $section->[0]->{'data'} } = grep(defined, @{ $section->[0]->{'data'} });
+
+            # If section happened to be empty, remove it as well
+            if (!scalar(@{ $section->[0]->{'data'} })) {
+                delete $settings[$sections];
+            } else {
+                push(@sections, [$section->[0]->{'id'}, $section->[0]->{'title'}]);
+            }
+        }
+    }
+    return { 'sections'            => \@sections,
+             'config_quick_access' => \@config_quick_access,
+             'settings'            => \@settings,
+             'excluded_options'    => \@excluded_options };
+}
+
 1;
