@@ -1,6 +1,11 @@
 # aix-lib.pl
 # Functions for aix package management
 
+sub list_package_system_commands
+{
+return ("lslpp", "geninstall", "installp");
+}
+
 # list_packages([package]*)
 # Fills the array %packages with a list of all packages
 sub list_packages
@@ -8,7 +13,8 @@ sub list_packages
 	local($_, $list, $i, $file, %fileset);
 	$i = 0;
 	$list = join(' ', @_);
-	open(LSLPP, "lslpp -L -c $list 2>&1 | grep -v '#' |");
+	%packages = ( );
+	&open_execute_command(LSLPP, "lslpp -L -c $list 2>&1 | grep -v '#'", 1, 1);
 	while($file = <LSLPP>) {
                 %fileset = &fileset_info($file); 
 		if ($fileset{"type"} eq "R") {
@@ -34,15 +40,16 @@ sub package_info
 {
 	local(@rv, @tmp, $d, $out, $archout, %fileset);
 
-	$out = `lslpp -L -c $_[0] 2>&1 | grep -v '#'`;
+	local $qm = quotemeta($_[0]);
+	$out = &backquote_command("lslpp -L -c $qm 2>&1 | grep -v '#'", 1);
 	%fileset = &fileset_info($out);
 	if ($out =~ /^lslpp:/) {
-		open(RPM, "rpm -q $_[0] --queryformat \"%{NAME}-%{VERSION}\\n%{GROUP}\\n%{ARCH}\\n%{VERSION}\\n%{VENDOR}\\n%{INSTALLTIME}\\n\" 2>/dev/null |");
+		&open_execute_command(RPM, "rpm -q $qm --queryformat \"%{NAME}-%{VERSION}\\n%{GROUP}\\n%{ARCH}\\n%{VERSION}\\n%{VENDOR}\\n%{INSTALLTIME}\\n\" 2>/dev/null", 1, 1);
 		@tmp = <RPM>;
 		chop(@tmp);
 		if (!@tmp) { return (); }
 		close(RPM);
-		open(RPM, "rpm -q $_[0] --queryformat \"%{DESCRIPTION}\" |");
+		&open_execute_command(RPM, "rpm -q $qm --queryformat \"%{DESCRIPTION}\"", 1, 1);
 		while(<RPM>) { $d .= $_; }
 		close(RPM);
 		return ($tmp[0], $tmp[1], $d, $tmp[2], $tmp[3], $tmp[4], &make_date($tmp[5]));
@@ -60,18 +67,27 @@ sub package_info
      	}
 }
 
+# virtual_package_info(package)
+# Returns an array of package information for a virtual package, usually called
+# if "package_info" returns nothing.
+sub virtual_package_info
+{
+return ( );
+}
+
 # is_package(file)
 # Tests if some file is a valid package file
 sub is_package
 {
 	local($out, $name, $filetype);
-	$out = `file $_[0] 2>&1`;
+	local $qm = quotemeta($_[0]);
+	$out = &backquote_command("file $qm 2>&1", 1);
 	($name, $filetype) = split(/:/, $out);
-	if ($filetype =~ /backup\/restore format file/) { 
+	if ($filetype =~ /backup\/restore format file/i) { 
 	 	$fileset{"filetype"} = "AIX";
 		return 1;	
   	}
-        elsif ($filetype =~ /RPM v3 bin PowerPC/) {
+        elsif ($filetype =~ /RPM v3 bin PowerPC/i) {
 		$fileset{"filetype"} = "RPM";
 		return 1;
         }
@@ -84,16 +100,18 @@ sub is_package
 sub file_packages
 {
         local($_, $line, $i, $j, $file, $out, $continue, @token, $firstline);
-	local(@output, $description, @vrmf, @field, $k, $l, $stub);
+	local(@rv, @output, $description, @vrmf, @field, $k, $l, $stub);
+	local $qm = quotemeta($_[0]);
+	local $real = &translate_filename($_[0]);
         if (&is_package($_[0])) {
                 @token = split(/\//, $_[0]);
                 $i = @token;
                 $file = $token[$i - 1];
 		if ($fileset{"filetype"} eq "RPM") {
 			$fileset{"fileset_name"} = "R:$file";
-			if (-d $_[0]) {
+			if (-d $real) {
 		       		local @rv;
-			        open(RPM, "cd $_[0] ; rpm -q -p *.rpm --queryformat \"%{NAME}-%{VERSION} %{SUMMARY}\\n\" 2>&1 |");
+			        &open_execute_file(RPM, "cd $qm ; rpm -q -p *.rpm --queryformat \"%{NAME}-%{VERSION} %{SUMMARY}\\n\" 2>&1", 1, 1);
 			        while(<RPM>) {
 			                chop;
 		        	        push(@rv, $_) if (!/does not appear|query of.*failed/);
@@ -103,12 +121,12 @@ sub file_packages
 		        }
 			else {
 			        local($out);
-		        	$out = `rpm -q -p $_[0] --queryformat "%{NAME}-%{VERSION} %{SUMMARY}" 2>&1`;
+		        	$out = &backquote_command("rpm -q -p $qm --queryformat \"%{NAME}-%{VERSION} %{SUMMARY}\" 2>&1", 1);
 			        return ($out);
 		        }
 		}
 		elsif ($fileset{"filetype"} eq "AIX") {
-			open(FILE, $_[0]);
+			&open_tempfile(FILE, $_[0]);
 			$line = <FILE>;
 			$continue = 1;
         	       	while ($continue) {
@@ -136,10 +154,11 @@ sub file_packages
 			}
 			@output = split(/\n/, $out);
 		        $j = @output;
-			$out = "$fileset{fileset_name}  ";
 			for ($i = 0; $i < $j; $i++) {
 				if ($output[$i] !~ /\{|\}|\*/) {
 					@field = split(/\s/, $output[$i]);
+                                        $out = undef;
+                                        $out = "$fileset{fileset_name}  ";
 					$description = undef;
 					@vrmf = split(/\./, $field[1]);
 					$vrmf[0] =~ s/^0+//;
@@ -151,12 +170,12 @@ sub file_packages
 						for ($l = 6; $l <= $k; $l++) {
 							$description = $description . " " . $field[$l];
 						}
-						$out = $out . "$field[0], $vrmf[0].$vrmf[1].$vrmf[2].$vrmf[3], 
-						       $description<br>";
+						$out = $out . "$field[0], $vrmf[0].$vrmf[1].$vrmf[2].$vrmf[3], $description";
 					}	
 				}
+                                push(@rv, $out);
 			}
-			return $out;
+			return @rv;
 		}
 	}
 	else {  return undef; }
@@ -333,12 +352,13 @@ sub install_package
 sub check_files
 {
 	local($_, $list, $i, $_, @w, %errs, %myfile, $epath, $path, $fileset, $file);
+	local $qm = quotemeta($_[0]);
 	$i = 0;
 	$list = join(' ', @_);
-	open(LSLPP, "lslpp -f -c $_[0] 2>&1 | grep -v '#' |");
+	&open_execute_command(LSLPP, "lslpp -f -c $qm 2>&1 | grep -v '#'", 1, 1);
 	$out = <LSLPP>;
 	if ($out =~ /lslpp:/) { 
-		open(RPM, "rpm -V $_[0] |");
+		&open_execute_command(RPM, "rpm -V $qm", 1, 1);
 		while(<RPM>) {
        			/^(.{8}) (.) (.*)$/;
 	        	if ($1 eq "missing ") { $errs{$3} = "Missing"; }
@@ -350,7 +370,7 @@ sub check_files
 	             	}
         	}
 		close(RPM);
-		open(RPM, "rpm -q $_[0] -l --dump |");
+		&open_execute_command(RPM, "rpm -q $qm -l --dump", 1, 1);
 		while(<RPM>) {
         		chop;
                 	if ($_ =~ /(contains no files)/) { return $i; }
@@ -358,7 +378,7 @@ sub check_files
        			$files{$i,'path'} = $w[0];
         		if ($w[10] ne "X") { $files{$i,'link'} = $w[10]; }
         		$files{$i,'type'} = $w[10] ne "X" ? 3 :
-					    (-d $w[0])    ? 1 :
+					    (-d &translate_filename($w[0]))? 1 :
 	                	            $w[7]         ? 5 : 0;
 	        	$files{$i,'user'} = $w[5];
         		$files{$i,'group'} = $w[6];
@@ -396,15 +416,16 @@ sub check_files
 sub installed_file
 {
 	local(%myfile);
-	$out = `lslpp -wc $_[0] 2>&1 | grep -v '#'`;
+	local $qm = quotemeta($_[0]);
+	$out = &backquote_command("lslpp -wc $qm 2>&1 | grep -v '#'", 1);
 	if ($?) {
 		local($pkg, @w, $_);
 		undef(%file);
-		$pkg = `rpm -q -f $_[0] --queryformat "%{NAME}-%{VERSION}\\n" 2>&1`;
+		$pkg = &backquote_command("rpm -q -f $qm --queryformat \"%{NAME}-%{VERSION}\\n\" 2>&1", 1);
 
 		if ($pkg =~ /not owned/ || $?) { return 0; }
 		@pkgs = split(/\n/, $pkg);
-		open(RPM, "rpm -q $pkgs[0] -l --dump |");
+		&open_execute_command(RPM, "rpm -q $pkgs[0] -l --dump", 1);
 		while(<RPM>) {
 		        chop;
 	        	@w = split(/ /);
@@ -413,7 +434,7 @@ sub installed_file
         	        	$file{'path'} = $w[0];
                 		if ($w[10] ne "X") { $files{$i,'link'} = $w[10]; }
              			$file{'type'} = $w[10] ne "X" ? 3 :
-                        	        	(-d $w[0])    ? 1 :
+					(-d &translate_filename($w[0])) ? 1 :
                                 		$w[7]         ? 5 : 0;
                 		$file{'user'} = $w[5];
                 		$file{'group'}= $w[6];
@@ -443,15 +464,15 @@ sub installed_file
 sub delete_package
 {
         local(%fileset, $file, $out, $rv);
-	local $temp = &tempname();
+	local $temp = &transname();
+	local $qm = quotemeta($_[0]);
 
-	$file = `lslpp -L -c $_[0] 2>&1 | grep -v '#'`;
+	$file = &backquote_command("lslpp -L -c $qm 2>&1 | grep -v '#'", 1);
 	%fileset = &fileset_info($file);
 
 	$rv = &system_logged("geninstall -u $_[0] >$temp 2>&1");
-	local $out = `cat $temp`;
-	$out = `cat $temp`;
-	unlink($temp);
+	local $out = &backquote_command("cat $temp");
+	&unlink_file($temp);
 	
 	if ($rv) { return "<pre>$out</pre>"; }
 	return undef; 
@@ -491,20 +512,24 @@ sub file_info
 	      $dev, $ino, $mode, $nlink, $uid, $gid, @args, $rdev,
 	      $size, $atime, $mtime, $ctime, $blksize, $blocks);
   	%file = ();	
+	local $real;
 	
         if ($_[0] =~ /->/) {
 		($filename, $symlink) = split(/->/, $_[0]);
+		$real = &translate_filename($filename);
 		chop($filename);
                 ($dev, $ino, $mode, $nlink, $uid, $gid, $rdev, $size,
-                 $atime, $mtime, $ctime, $blksize, $blocks) = lstat $filename;
+                 $atime, $mtime, $ctime, $blksize, $blocks) = lstat($real);
 	}
 	else {
 		$filename = $_[0];
+		$real = &translate_filename($filename);
                 ($dev, $ino, $mode, $nlink, $uid, $gid, $rdev, $size,
-                 $atime, $mtime, $ctime, $blksize, $blocks) = stat $filename;
+                 $atime, $mtime, $ctime, $blksize, $blocks) = stat($real);
 	}
 	
-	@out = `lslpp -wc $filename 2>&1`;
+	local $qm = quotemeta($filename);
+	@out = &backquote_command("lslpp -wc $qm 2>&1", 1);
 	($stub, $fileset, $type) = split(/:/, $out[1]);
 
         $file{"path"}    = $filename;	
@@ -521,18 +546,23 @@ sub file_info
 	$file{"ctime"}   = scalar(localtime($ctime));
 	$file{"blksize"} = $blksize;
 	$file{"blocks"}  = $blocks;
-	$file{"link"}    = readlink($filename);
+	$file{"link"}    = readlink($real);
 	$file{"package"} = $fileset;
-	$file{"err"}     = -e $filename  ? 0 : "Error";
-	$file{"type"}    = -l $filename ? 3 :
-                           -d $filename ? 1 :
+	$file{"err"}     = -e $real  ? 0 : "Error";
+	$file{"type"}    = -l $real ? 3 :
+                           -d $real ? 1 :
 			   0;
 	return(%file);
 }
 
 sub package_system
 {
-	return "AIX Package Manager";
+	return $text{'aix_manager'}; 
+}
+
+sub package_help
+{
+        return "installp lslpp";
 }
 
 1;
