@@ -13,13 +13,16 @@ sub list_packages
 local($i, $list); $i = 0;
 $list = @_ ? join(' ', map { quotemeta($_) } @_) : "-a";
 %packages = ( );
-&open_execute_command(RPM, "rpm -q $list --queryformat \"%{NAME}\\n%{VERSION}-%{RELEASE}\\n%{EPOCH}\\n%{GROUP}\\n%{SUMMARY}\\n\\n\"", 1, 1);
-while($packages{$i,'name'} = <RPM>) {
+&open_execute_command(RPM, "rpm -q $list --queryformat \"%{NAME}\\n%{VERSION}-%{RELEASE}\\n%{EPOCH}\\n%{GROUP}\\n%{ARCH}\\n%{SUMMARY}\\n\\n\"", 1, 1);
+while(my $name = <RPM>) {
+	next if ($name =~ /is\s+not\s+installed$/);
+	$packages{$i,'name'} = $name;
 	chop($packages{$i,'name'});
 	chop($packages{$i,'version'} = <RPM>);
 	chop($packages{$i,'epoch'} = <RPM>);
 	$packages{$i,'epoch'} = undef if ($packages{$i,'epoch'} eq '(none)');
 	chop($packages{$i,'class'} = <RPM>);
+	chop($packages{$i,'arch'} = <RPM>);
 	while(<RPM>) {
 		s/\r|\n/ /g;
 		last if (!/\S/);
@@ -55,6 +58,29 @@ close(RPM);
 return ($tmp[0], $tmp[1], $d, $tmp[2], $tmp[3], $tmp[4], &make_date($tmp[5]));
 }
 
+# virtual_package_info(package)
+# Returns an array of package information for a virtual package, usually called
+# if "package_info" returns nothing.
+sub virtual_package_info
+{
+my ($vpkg) = @_;
+my $query_format =
+	'%{NAME}\n%{GROUP}\n%{ARCH}\n%{VERSION}-%{RELEASE}\n%{VENDOR}\n'.
+	'%{INSTALLTIME}\n%{DESCRIPTION}';
+
+&open_execute_command(*RPM, "rpm -q --whatprovides $vpkg --queryformat ".
+			    "\"$query_format\" 2>/dev/null", 1, 1);
+my @tmp = <RPM>;
+close(RPM);
+
+return () if (!@tmp || $tmp[0] =~ /no\s+package\s+provides/);
+
+# Extract fields properly
+my @fields = splice(@tmp, 0, 6);
+chomp(@fields);
+return (@fields[0, 1], join('', @tmp), @fields[2, 3, 4], &make_date($fields[5]));
+}
+
 # is_package(file)
 # Check if some file is a package file
 sub is_package
@@ -108,7 +134,7 @@ if (-d $real) {
 	}
 elsif ($_[0] =~ /\*|\?/) {
 	local @rv;
-	&open_execute_command(RPM, "rpm -q -p $_[0] --queryformat \"%{NAME} %{SUMMARY}\\n\" 2>&1", 1);
+	&open_execute_command(RPM, "rpm -q -p $qm --queryformat \"%{NAME} %{SUMMARY}\\n\" 2>&1", 1);
 	while(<RPM>) {
 		chop;
 		push(@rv, $_) if (!/does not appear|query of.*failed|warning:/);
@@ -392,6 +418,51 @@ if ($? || $out =~ /error:/) { return "<pre>$out</pre>"; }
 return undef;
 }
 
+# package_dependencies(name, [version])
+# Returns a list of packages that this one depends on, as hash refs
+sub package_dependencies
+{
+my ($pkg, $ver) = @_;
+my $out = &backquote_command(
+	"rpm -qR ".quotemeta($pkg.($ver ? "-$ver" : ""))." 2>/dev/null");
+my @rv;
+my %done;
+foreach my $l (split(/\r?\n/, $out)) {
+	next if ($done{$l}++);
+	if ($l =~ /^(\/.*)$/) {
+		# Depends on a file
+		push(@rv, { 'file' => $1 });
+		}
+	elsif ($l =~ /^([^\(\)= ]+)$/) {
+		# A bare package name
+		push(@rv, { 'package' => $1 });
+		}
+	elsif ($l =~ /^([^\(\)= ]+)\s+([=<>]+)\s+(\S+)$/) {
+		# A package name and matching version
+		push(@rv, { 'package' => $1,
+			    'compare' => $2,
+			    'version' => $3 });
+		}
+	elsif ($l =~ /^(\S+)\s+([=<>]+)\s+(\S+)$/) {
+		# Some other capability and matching version
+		push(@rv, { 'other' => $1,
+			    'compare' => $2,
+			    'version' => $3 });
+		}
+	elsif ($l =~ /^(\S+)\((\S*)\)\((\S*)\)$/) {
+		# Library with a tag and architecture
+		push(@rv, { 'library' => $1,
+			    'version' => $2,
+			    'arch' => $3 });
+		}
+	elsif ($l =~ /^(\S+)$/) {
+		# Some other capability
+		push(@rv, { 'other' => $1 });
+		}
+	}
+return @rv;
+}
+
 sub package_system
 {
 return "RPM";
