@@ -33,6 +33,11 @@ else {
 		&set_config_file($access{'file'});
 		}
 	else {
+		if (!-r $config{'local_cf'} && -r $config{'alt_local_cf'}) {
+			# Copy in default config file
+			&copy_source_dest($config{'alt_local_cf'},
+					  $config{'local_cf'});
+			}
 		&set_config_file($config{'local_cf'});
 		}
 	if ($access{'nocheck'}) {
@@ -113,7 +118,7 @@ if ($include_config_files || $forglobal) {
 		}
 	else {
 		# A single file that can be read right here
-		open(FILE, $file);
+		open(FILE, "<".$file);
 		while(<FILE>) {
 			s/\r|\n//g;
 			s/^#.*$//;
@@ -177,20 +182,45 @@ elsif ($config{'mode'} == 3 && !$forglobal) {
 return \@rv;
 }
 
-# find(name, &config)
+# alt_name(name, &config)
+# Returns an alternate directive name if the current one is obsoleted
+sub alt_name
+{
+my ($name, $conf) = @_;
+my $newname = $name;
+if ($newname =~ s/whitelist/welcomelist/ ||
+    $newname =~ s/blacklist/blocklist/) {
+	# Is the new directive being used?
+	my @olduse = &find($name, $conf, 1);
+	my @newuse = &find($newname, $conf, 1);
+	if (@newuse) {
+		# New name exists in config, so use it
+		$name = $newname;
+		}
+	elsif (!@olduse && &version_atleast("4.0")) {
+		# Neither exists, but use the new name is later versions
+		$name = $newname;
+		}
+	}
+return $name;
+}
+
+# find(name, &config, [no-alt])
 sub find
 {
-local @rv;
-foreach $c (@{$_[1]}) {
-	push(@rv, $c) if (lc($c->{'name'}) eq lc($_[0]));
+my ($name, $conf, $noalt) = @_;
+my @rv;
+$name = &alt_name($name, $conf) if (!$noalt);
+foreach my $c (@$conf) {
+	push(@rv, $c) if (lc($c->{'name'}) eq lc($name));
 	}
 return wantarray ? @rv : $rv[0];
 }
 
-# find_value(name, &config)
+# find_value(name, &config, [no-alt])
 sub find_value
 {
-local @rv = map { $_->{'value'} } &find(@_);
+my @rv = map { $_->{'value'} } &find(@_);
 return wantarray ? @rv : $rv[0];
 }
 
@@ -198,6 +228,7 @@ return wantarray ? @rv : $rv[0];
 # Update the config file with some directives
 sub save_directives
 {
+my ($conf, $nameold, $newv, $vonly) = @_;
 if ($module_info{'usermin'} && $local_cf =~ /^(.*)\/([^\/]+)$/) {
 	# Under Usermin, make sure .spamassassin exists
 	local $spamdir = $1;
@@ -205,11 +236,13 @@ if ($module_info{'usermin'} && $local_cf =~ /^(.*)\/([^\/]+)$/) {
 		&make_dir($spamdir, 0755);
 		}
 	}
-local @old = ref($_[1]) ? @{$_[1]} : &find($_[1], $_[0]);
-local @new = $_[3] ? &make_directives($_[1], $_[2]) : @{$_[2]};
-local $i;
-for($i=0; $i<@old || $i<@new; $i++) {
-	local $line;
+if (!ref($nameold)) {
+	$nameold = &alt_name($nameold, $conf);
+	}
+my @old = ref($nameold) ? @$nameold : &find($nameold, $conf);
+my @new = $vonly ? &make_directives($nameold, $newv) : @$newv;
+for(my $i=0; $i<@old || $i<@new; $i++) {
+	my $line;
 	if ($new[$i]) {
 		$line = $new[$i]->{'name'};
 		$line .= " ".$new[$i]->{'value'} if ($new[$i]->{'value'} ne '');
@@ -257,7 +290,7 @@ for($i=0; $i<@old || $i<@new; $i++) {
 				    $rv ? $rv->error : "Unknown modify error"));
 				}
 			}
-		$_[0]->[$old[$i]->{'index'}] = $new[$i];
+		$conf->[$old[$i]->{'index'}] = $new[$i];
 		}
 	elsif ($old[$i]) {
 		# Deleting a directive
@@ -265,7 +298,7 @@ for($i=0; $i<@old || $i<@new; $i++) {
 			# From a file
 			local $lref = &read_file_lines($old[$i]->{'file'});
 			splice(@$lref, $old[$i]->{'line'}, 1);
-			foreach $c (@{$_[0]}) {
+			foreach $c (@$conf) {
 				if ($c->{'line'} > $old[$i]->{'line'} &&
 				    $c->{'file'} eq $old[$i]->{'file'}) {
 					$c->{'line'}--;
@@ -303,8 +336,8 @@ for($i=0; $i<@old || $i<@new; $i++) {
 			}
 
 		# Fix up indexes
-		splice(@{$_[0]}, $old[$i]->{'index'}, 1);
-		foreach $c (@{$_[0]}) {
+		splice(@$conf, $old[$i]->{'index'}, 1);
+		foreach $c (@$conf) {
 			if ($c->{'index'} > $old[$i]->{'index'}) {
 				$c->{'index'}--;
 				}
@@ -348,17 +381,19 @@ for($i=0; $i<@old || $i<@new; $i++) {
 				}
 			}
 		$new[$i]->{'mode'} = $addmode;
-		$new[$i]->{'index'} = @{$_[0]};
-		push(@{$_[0]}, $new[$i]);
+		$new[$i]->{'index'} = @$conf;
+		push(@$conf, $new[$i]);
 		}
 	}
 }
 
 # make_directives(name, &values)
+# Converts a list of values into a list of directive objects
 sub make_directives
 {
-return map { { 'name' => $_[0],
-	       'value' => $_ } } @{$_[1]};
+my ($name, $vals) = @_;
+return map { { 'name' => $name,
+	       'value' => $_ } } @$vals;
 }
 
 ### UI functions ###
@@ -572,7 +607,7 @@ sub version_atleast
 if (!$version_cache) {
 	$version_cache = &get_spamassassin_version();
 	}
-return $version_cache >= $_[0];
+return &compare_version_numbers($version_cache, '>=', $_[0]);
 }
 
 # spam_file_folder()
@@ -966,6 +1001,8 @@ return $uinfo;
 sub get_auto_whitelist_file
 {
 local ($user) = @_;
+local @uinfo = $module_info{'usermin'} ? @remote_user_info :
+	       $user ? getpwnam($user) : ( );
 local $conf = &get_config();
 local $awp = &find("auto_whitelist_path", $conf);
 if (!$awp) {
@@ -974,8 +1011,6 @@ if (!$awp) {
 $awp ||= "~/.spamassassin/auto-whitelist";
 if ($awp !~ /^\//) {
 	# Make absolute
-	local @uinfo = $module_info{'usermin'} ? @remote_user_info :
-		       $user ? getpwnam($user) : ( );
 	return undef if (scalar(@uinfo) == 0);
 	$awp =~ s/^(\~|\$HOME)\//$uinfo[7]\//;
 	if ($awp !~ /^\//) {
@@ -987,6 +1022,10 @@ if (!-r $awp) {
 	local @real = glob("$awp.*");
 	$awp = undef if (!@real);
 	}
+# Is it under the user's home?
+if (!&is_under_directory($uinfo[7], $awp)) {
+	$awp = undef;
+	}
 return $awp;
 }
 
@@ -1064,7 +1103,7 @@ else {
 sub list_spamassassin_languages
 {
 local @rv;
-open(LANGS, "$module_root_directory/langs");
+open(LANGS, "<$module_root_directory/langs");
 while(<LANGS>) {
 	if (/^(\S+)\s+(.*)/) {
 		push(@rv, [ $1, $2 ]);
@@ -1079,7 +1118,7 @@ return @rv;
 sub list_spamassassin_locales
 {
 local @rv;
-open(LANGS, "$module_root_directory/locales");
+open(LANGS, "<$module_root_directory/locales");
 while(<LANGS>) {
 	if (/^(\S+)\s+(.*)/) {
 		push(@rv, [ $1, $2 ]);
@@ -1103,5 +1142,57 @@ push(@rv, &find_value("loadplugin", $conf));
 return @rv;
 }
 
+# get_procmail_status()
+# Returns flags indicating if spamassassin is called, and if delivery based
+# on the headers it adds are enabled, based on the procmail config.
+sub get_procmail_status
+{
+if (!$warn_procmail || !&foreign_check("procmail")) {
+	# Don't know, or checking disabled
+	return (-1, -1);
+	}
+&foreign_require("procmail");
+my $spam_enabled = 0;		# Found call to spamassassin
+my $delivery_enabled = 0;	# Found X-Spam: header rule
+my @pmrcs = &get_procmailrc();
+foreach my $pmrc (@pmrcs) {
+	my @recipes = &procmail::parse_procmail_file($pmrc);
+	my $isglobal = $pmrc eq $config{'global_procmailrc'} ||
+		       $pmrc eq $config{'procmailrc'} ||
+		       $pmrc eq $procmail::procmailrc;
+	if (&find_spam_recipe(\@recipes)) {
+		$spam_enabled ||= 1;
+		}
+	if (&find_file_recipe(\@recipes)) {
+		if ($isglobal) {
+			# Enabled globally, and so
+			# cannot be changed by user
+			$delivery_enabled ||= -2;
+			}
+		else {
+			$delivery_enabled ||= 1;
+			}
+		}
+	if (&find_virtualmin_recipe(\@recipes) ||
+	    &foreign_installed("virtual-server")) {
+		# Controlled by Virtualmin
+		if ($isglobal &&
+		    &find_force_default_receipe(
+		      \@recipes)) {
+			# User .procmailrc files are
+			# prevented
+			$spam_enabled ||= -2;
+			$delivery_enabled ||= -2;
+			}
+		else {
+			# Users can have a .procmailrc
+			$spam_enabled ||= -2;
+			$delivery_enabled ||= 1;
+			}
+		}
+	}
+return ($spam_enabled, $delivery_enabled);
+}
+
 1;
 
